@@ -1,6 +1,6 @@
 /*
  *  KSeg
- *  Copyright (C) 1999-2001 Ilya Baran
+ *  Copyright (C) 1999-2003 Ilya Baran
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,11 +26,24 @@
 
 #include "KSegDocument.H"
 #include "KSegConstruction.H"
+#include "KSegProperties.H"
 #include "G_ref.H"
 #include "G_object.H"
 #include "G_refSearcher.H"
+#include "KSegSelectionGroup.H"
 
 
+KSegDocument::KSegDocument() : isUndoing(false), isRedoing(false),
+		 undoStack(this), redoStack(this), changed(false)
+{
+  connect(this, SIGNAL(documentChanged()), this, SLOT(IChanged()));
+  printer = new QPrinter();
+  printer->setOrientation(QPrinter::Landscape);
+  defaultDrawstyle = G_drawstyle::defaultStyle();
+  setCurrentColor(QColor(KSegProperties::getProperty("ForegroundColor")));
+  numViews = 0;
+}
+
 KSegDocument::~KSegDocument()
 {
   delete printer;
@@ -63,8 +76,31 @@ void KSegDocument::delRef(G_ref *inRef)
   emitDocumentModified();
 }
 
-void KSegDocument::draw(QPainter &p)
+
+struct VT {
+  VT(G_refs &r, const ViewTransform &inTrans) : refs(r), trans(inTrans) {
+    for(int i = 0; i < (int)refs.size(); ++i) {
+      if(refs[i]->isDrawn()) refs[i]->getObject()->tempTransform(trans);
+    }
+  }
+  
+  ~VT() {
+    ViewTransform inv(-trans.offsetX * trans.zoom, -trans.offsetY * trans.zoom, 1. / trans.zoom);
+    
+    for(int i = 0; i < (int)refs.size(); ++i) {
+      if(refs[i]->isDrawn()) refs[i]->getObject()->tempTransform(inv);
+    }
+  }
+
+private:
+  G_refs &refs;
+  ViewTransform trans;
+};
+
+
+void KSegDocument::draw(QPainter &p, const ViewTransform &trans)
 {
+  VT tempTrans(allRefs, trans);
   int i;
 
   //first draw the markings on marked objects:
@@ -264,8 +300,9 @@ void KSegDocument::draw(QPainter &p)
 }
 
 
-void KSegDocument::print(QPainter &p)
+void KSegDocument::print(QPainter &p, const ViewTransform &trans)
 { //kind of like draw, but doesn't draw markings or selection.
+  VT tempTrans(allRefs, trans);
   int i;
 
   //first draw the labels as if unselected:
@@ -327,8 +364,9 @@ void KSegDocument::afterLoadDraw()
 
 
 
-QRect KSegDocument::getSize()
+QRect KSegDocument::getSize(const ViewTransform &trans)
 {
+  VT tempTrans(allRefs, trans);
   int i;
   QRect tmp = QRect();
 
@@ -341,8 +379,9 @@ QRect KSegDocument::getSize()
   return tmp;
 }
 
-G_refs KSegDocument::whatLabelsAmIOn(int x, int y)
+G_refs KSegDocument::whatLabelsAmIOn(int x, int y, const ViewTransform &trans)
 {
+  VT tempTrans(allRefs, trans);
   int i;
   G_refs tmp;
 
@@ -357,8 +396,9 @@ G_refs KSegDocument::whatLabelsAmIOn(int x, int y)
   return tmp;
 }
 
-G_refs KSegDocument::whatAmIOn(int x, int y, bool pointsOnly)
+G_refs KSegDocument::whatAmIOn(int x, int y, const ViewTransform &trans, bool pointsOnly)
 {
+  VT tempTrans(allRefs, trans);
   int i;
   G_refs tmp;
 
@@ -405,8 +445,9 @@ G_refs KSegDocument::whatAmIOn(int x, int y, bool pointsOnly)
   return tmp;
 }
 
-void KSegDocument::addSelect(const QRect &r)
+void KSegDocument::addSelect(const QRect &r, const ViewTransform &trans)
 {
+  VT tempTrans(allRefs, trans);
   int i;
 
   for(i = 0; i < (int)allRefs.count(); i++) {
@@ -973,10 +1014,10 @@ QDataStream &operator<<(QDataStream &streamx, KSegDocument &doc)
   int i;
 
   if(doc.isConstruction()) {
-    streamx << QString("KSeg Construction Version 0.1");
+    streamx << QString("KSeg Construction Version 0.2");
 
-    //save some values so that constructions can be quickly extracted and
-    //applied
+    //save the given types so that constructions can be quickly extracted and
+    //it's easy to check if they can be applied
     KSegConstruction *d = (KSegConstruction *)(&doc);
     streamx << d->given.size();
     for(i = 0; i < (int)d->given.size(); ++i) {
@@ -984,7 +1025,7 @@ QDataStream &operator<<(QDataStream &streamx, KSegDocument &doc)
     }
   }
   else {
-    streamx << QString("KSeg Document Version 0.0");
+    streamx << QString("KSeg Document Version 0.2");
   }
 
   QByteArray array;
@@ -992,20 +1033,20 @@ QDataStream &operator<<(QDataStream &streamx, KSegDocument &doc)
 
   buf.open(IO_WriteOnly);
   QDataStream stream(&buf);
-
+  stream.setVersion(3);
 
   QMap<G_drawstyle *, short> styleMap = G_drawstyle::saveUsed(stream, doc.allRefs);
 
   stream << doc.allRefs.count();
 
-  QMap<G_ref *, int> refMap;
+  hash_map<G_ref *, int> refMap;
 
   doc.allRefs.topologicalSort(doc.allRefs);    
 
   //this assumes all the stuff is topologically sorted
 
   for(i = 0; i < (int)doc.allRefs.count(); i++) {
-    refMap.insert(doc.allRefs[i], i);
+    refMap[doc.allRefs[i]] = i;
 
     stream << styleMap[doc.allRefs[i]->getDrawstyle()];
 
@@ -1026,19 +1067,22 @@ QDataStream &operator<<(QDataStream &streamx, KSegDocument &doc)
     }
   }
 
+  //now save the selection groups
+  int selGroupCount = 0;
+  for(i = 0; i < (int)doc.selectionGroups.size(); ++i) {
+    if(doc.selectionGroups[i]->size() > 0) selGroupCount++;
+  }
+
+  stream << selGroupCount;
+  for(i = 0; i < (int)doc.selectionGroups.size(); ++i) {
+    if(doc.selectionGroups[i]->size() > 0) doc.selectionGroups[i]->save(stream, refMap);
+  }
+
+  //done saving data
+
   buf.close();
 
-  streamx << array.size();
-
-  QByteArray compressed((array.size() * 1001) / 1000 + 15);
-
-  unsigned long destLen = compressed.size();
-
-  compress2((Bytef *)compressed.data(), &destLen, (Bytef *)array.data(), array.size(), 9);
-
-  compressed.resize(destLen);
-
-  streamx << compressed;
+  streamx << array;
 
   return streamx;
 }
@@ -1050,7 +1094,12 @@ QDataStream &operator>>(QDataStream &streamx, KSegDocument &doc)
 
   streamx >> version;
 
-  if(version == QString("KSeg Construction Version 0.1")) {
+  int majVer, minVer;
+  bool isConstr;
+
+  KSegDocument::parseVersionString(version, &isConstr, &majVer, &minVer);
+
+  if(isConstr && (majVer > 0 || minVer > 0)) {
     //ignore the information for quick check:
     unsigned int i;
     int dummy;
@@ -1059,24 +1108,32 @@ QDataStream &operator>>(QDataStream &streamx, KSegDocument &doc)
     while(i--) streamx >> dummy;
   }
 
-  unsigned long origSize;
-  int origTmp;
+  QByteArray array;
 
-  streamx >> origTmp;
-  origSize = origTmp;
+  if(majVer == 0 && minVer < 2) { //old files come compressed
+    unsigned long origSize;
+    int origTmp;
 
-  QByteArray compressed;
+    streamx >> origTmp;
+    origSize = origTmp;
 
-  streamx >> compressed;
-  
-  QByteArray array(origSize + 2);
+    QByteArray compressed;
 
-  uncompress((Bytef *)array.data(), &origSize, (Bytef *)compressed.data(), compressed.size());
+    streamx >> compressed;
+    
+    array.resize(origSize + 2);
+
+    uncompress((Bytef *)array.data(), &origSize, (Bytef *)compressed.data(), compressed.size());
+  }
+  else { //now space is cheap
+    streamx >> array;
+  }
 
   QBuffer buf(array);
 
   buf.open(IO_ReadOnly);
   QDataStream stream(&buf);
+  stream.setVersion(3);
 
   QArray<G_drawstyle *> styleArray = G_drawstyle::loadUsed(stream);
 
@@ -1114,7 +1171,7 @@ QDataStream &operator>>(QDataStream &streamx, KSegDocument &doc)
     //to any objects.  Now is the time to delete these extraneous references.
   }
 
-  if(doc.isConstruction()) { //load the order of the givens
+  if(isConstr) { //load the order of the givens
     KSegConstruction *d = (KSegConstruction *)(&doc);
     
     int numGivens;
@@ -1127,6 +1184,16 @@ QDataStream &operator>>(QDataStream &streamx, KSegDocument &doc)
     }
   }  
 
+  if(majVer > 0 || minVer > 1) { //load selection groups
+    int numGroups;
+    stream >> numGroups;
+    doc.selectionGroups.resize(numGroups);
+    for(i = 0; i < numGroups; ++i) {
+      doc.selectionGroups[i] = new KSegSelectionGroup(&doc);
+      doc.selectionGroups[i]->load(stream);
+    }
+  }
+
   buf.close();
   
   doc.allRefs.update();
@@ -1138,4 +1205,34 @@ QDataStream &operator>>(QDataStream &streamx, KSegDocument &doc)
   return streamx;
 }
 
+
+//parses a string like "KSeg Document Version 2.3" and returns true if successful
+bool KSegDocument::parseVersionString(const QString &s, bool *isConstruction,
+				      int *majorVersion, int *minorVersion)
+{
+  if(s.startsWith("KSeg Document Version ") &&
+     s.length() == QString("KSeg Document Version 0.0").length()) {
+    if(isConstruction != NULL) *isConstruction = false;
+  }
+  else if(s.startsWith("KSeg Construction Version ") &&
+     s.length() == QString("KSeg Construction Version 0.0").length()) {
+    if(isConstruction != NULL) *isConstruction = true;
+  }
+  else return false;
+
+  bool ok;
+  int minV, majV;
+  
+  minV = s.right(1).toInt(&ok);
+  if(!ok) return false;
+  
+  majV = s.right(3).left(1).toInt(&ok);
+  if(!ok) return false;
+  
+  if(minorVersion != NULL) *minorVersion = minV;
+  if(majorVersion != NULL) *majorVersion = majV;
+
+  return true;
+}
+
 #include "KSegDocument.moc"
