@@ -1,6 +1,6 @@
 /*
  *  KSeg
- *  Copyright (C) 1999-2001 Ilya Baran
+ *  Copyright (C) 1999-2003 Ilya Baran
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 
 #include <qscrollbar.h>
 #include <qapplication.h>
+#include <qcursor.h>
+#include <qbitmap.h>
+#include <qdatetime.h>
 
 #include "KSegView.H"
 #include "G_point.H"
@@ -38,6 +41,8 @@
 #include "G_refSearcher.H"
 #include "KSegProperties.H"
 
+#include "pics/zoom_cursor.xpm"
+
 
 //the following class and the function after are for determining if A can be
 //reconstrained to B.
@@ -76,7 +81,7 @@ bool canReconstrain(G_ref *A, G_ref *B)
 {
   if(B->getType() & (G_POINT | G_CURVE) == 0) return false;
 
-  if(B->getDocument()->isConstruction() && B->getType() == G_POINT && B->getGiven() &&
+  if(B->getDocument()->isConstruction() && B->getType() == G_POINT && A->getGiven() &&
      ((KSegConstruction *)(B->getDocument()))->hasLoops()) return false;
 
   ATypeFinder atf(B);
@@ -84,7 +89,7 @@ bool canReconstrain(G_ref *A, G_ref *B)
 
   if(atf.type == ATypeFinder::Normal || B->getDocument()->isConstruction() == false) return true;  
 
-  KSegConstruction *c = (KSegConstruction *)(B->getDocument()); //I am sure--look above.
+  KSegConstruction *c = (KSegConstruction *)(B->getDocument());
 
   ImplicitInitialSearcher iis(A);
   if(iis.search(B) == false) {
@@ -117,8 +122,9 @@ bool canReconstrain(G_ref *A, G_ref *B)
 KSegView::SelectType KSegView::selectType;
 QList<KSegView> KSegView::allViews;
 QPainter *KSegView::constructionPainter;
+ViewTransform KSegView::constructionTransform;
 
-const int KSegView::ScrollBarThickness = 20;
+int KSegView::ScrollBarThickness = 20;
 
 KSegView::KSegView(KSegDocument *inDoc, QWidget *parent)
   : QWidget(parent)
@@ -126,13 +132,16 @@ KSegView::KSegView(KSegDocument *inDoc, QWidget *parent)
   allViews.append(this);
 
   menusEnabled = true;
+  nextZoom = false;
+  nextPan = false;
 
   doc = inDoc;
 
   doc->addView();
+  docReferenceRemoved = false;
 
   connect(doc, SIGNAL(documentChanged()), this, SLOT(redrawBuffer()));
-  connect(doc, SIGNAL(documentModified()), this, SLOT(documentChanged()));
+  connect(doc, SIGNAL(documentModified()), this, SLOT(documentModified()));
   connect(doc, SIGNAL(documentSaved(const QString &)),
 	  this, SLOT(documentSaved(const QString &)));
 
@@ -151,11 +160,14 @@ KSegView::KSegView(KSegDocument *inDoc, QWidget *parent)
   hBar = new QScrollBar(QScrollBar::Horizontal, this);
   vBar = new QScrollBar(QScrollBar::Vertical, this);
 
+  if(ScrollBarThickness == 20) ScrollBarThickness = hBar->sizeHint().height();
+
   setMouseTracking(true);
 
   drag = NO_DRAG;
 
   offsetX = offsetY = 0;
+  zoom = 1;
 
   connect(hBar, SIGNAL(valueChanged(int)), this, SLOT(moved(int)));
   connect(vBar, SIGNAL(valueChanged(int)), this, SLOT(moved(int)));
@@ -163,22 +175,39 @@ KSegView::KSegView(KSegDocument *inDoc, QWidget *parent)
   setBackgroundMode(NoBackground);
 
   connect(&selectTimer, SIGNAL(timeout()), this, SLOT(selectTimeout()));
-
 }
 
-
+//document deletion works like this: if we're not deleting the last
+//view (there are other views pointing to the document), we call
+//delView in delDocumentRef (to decrease the reference count) and set
+//docReferenceRemoved to mark that the reference count of the document
+//has been decreased.  Otherwise, we only delete the document when it
+//comes time to destroy the view.
 KSegView::~KSegView()
 {
-  doc->delView();
+  if(!docReferenceRemoved) doc->delView();
   allViews.removeRef(this);
 }
 
+void KSegView::delDocumentRef()
+{
+  if(doc->getNumViews() > 1) {
+    doc->delView();
+    docReferenceRemoved = true;
+  }
+}
 
+static bool inMoved = false; //recursion prevention
 void KSegView::moved(int)
 {
-  offsetX = hBar->value();
-  offsetY = vBar->value();
+  if(inMoved) return;
+  inMoved = true;
+
+  offsetX -= ((double)hBar->value()) / zoom;
+  offsetY -= ((double)vBar->value()) / zoom;
+
   redrawBuffer();
+  inMoved = false;
 }
 
 void KSegView::resizeEvent(QResizeEvent *e)
@@ -197,7 +226,9 @@ void KSegView::redrawBuffer()
 {
   QPainter p;
 
-  emit updateMenus();
+  if(drag == NO_DRAG || (drag != LABEL_DRAG && objectsUpdating.count() == 0)) {
+    if(drag != ZOOM_DRAG && drag != PAN_DRAG && !inMoved) emit updateMenus();
+  }
 
   if(selectType == BLINKING_SELECT) {
     if(doc->getSelected().count() > 0) {
@@ -207,15 +238,13 @@ void KSegView::redrawBuffer()
       if(selectTimer.isActive()) selectTimer.stop();
     }
   }
+  else if(selectTimer.isActive()) selectTimer.stop();
 
-  buffer.fill();
+  buffer.fill(QColor(KSegProperties::getProperty("BackgroundColor")));
 
   p.begin(&buffer);
-  QRect window = p.viewport();
-  window.moveBy(offsetX, offsetY);
-  p.setWindow(window);
 
-  doc->draw(p);
+  doc->draw(p, ViewTransform(offsetX, offsetY, zoom));
 
   p.end();
 
@@ -224,6 +253,8 @@ void KSegView::redrawBuffer()
 
 void KSegView::paintEvent(QPaintEvent *)
 {
+  if(doc == NULL) return;
+
   if(drag == RECTANGLE_DRAG) {
     QPainter p(&buffer);
 
@@ -237,14 +268,17 @@ void KSegView::paintEvent(QPaintEvent *)
 
   }
   else {
-    QRect tmp = doc->getSize();
+    QRect tmp = doc->getSize(ViewTransform(offsetX, offsetY, zoom));
     
-    if(tmp.isEmpty() || tmp.isNull()) tmp = QRect(offsetX + 1, offsetY + 1, 2, 2);
-    
-    hBar->setRange(QMIN(offsetX, tmp.left()), QMAX(offsetX, tmp.right() - width() + ScrollBarThickness));
+    if(tmp.isEmpty() || tmp.isNull()) tmp = QRect(1, 1, 2, 2);
+
+    hBar->setRange(QMIN(0, tmp.left()), QMAX(0, tmp.right() - width() + ScrollBarThickness));
     hBar->setSteps(10, width() - ScrollBarThickness);
-    vBar->setRange(QMIN(offsetY, tmp.top()), QMAX(offsetY, tmp.bottom() - height() + ScrollBarThickness));
+    hBar->setValue(0);
+
+    vBar->setRange(QMIN(0, tmp.top()), QMAX(0, tmp.bottom() - height() + ScrollBarThickness));
     vBar->setSteps(10, height() - ScrollBarThickness);
+    vBar->setValue(0);
 
     bitBlt(this, 0, 0, &buffer);
   }
@@ -257,28 +291,37 @@ void KSegView::documentSaved(const QString &filename)
   topLevelWidget()->setCaption(QString("kseg: ") + filename);
 }
 
-void KSegView::documentChanged()
+void KSegView::documentModified()
 {
   QString caption = "kseg: ";
   if(getDocument()->getFilename().isEmpty()) caption += tr("Untitled");
   else caption += getDocument()->getFilename();
   caption += '*';
-  
+
   if(topLevelWidget()->caption() != caption) topLevelWidget()->setCaption(caption);
 }
 
 //---------------------MOUSE PRESS EVENT-----------------------------
 void KSegView::mousePressEvent(QMouseEvent *e)
 {
+  if(doc == NULL) return;
+
   dragCurX = dragStartX = lastMouseX = e->x();
   dragCurY = dragStartY = lastMouseY = e->y();
 
   if(drag != NO_DRAG) return;
 
-  if(e->button() == LeftButton) {//----------------LEFT BUTTON PRESSED
-    G_refs tmp = doc->whatAmIOn(e->x() + offsetX, e->y() + offsetY);
+  if(nextPan || e->button() == MidButton) {
+    drag = PAN_DRAG; if(!nextPan) { nextPan = true; updateStatusBar(NoButton); }
+    emit updateMenus();
+    return;
+  }
+  if(nextZoom) { drag = ZOOM_DRAG; emit updateMenus(); return; }
 
-    if(e->state() & ControlButton) { //CTRL pressed--reconstrain
+  if(e->button() == LeftButton) {//----------------LEFT BUTTON PRESSED
+    G_refs tmp = doc->whatAmIOn(e->x(), e->y(), ViewTransform(offsetX, offsetY, zoom));
+
+    if(e->state() & ControlButton && !(e->state() & ShiftButton)) { //CTRL pressed--reconstrain
       if(tmp.count() == 0 || tmp[0]->getType() != G_POINT) return;
 
       if(!doc->isPointFreeable(tmp[0])) return;
@@ -301,8 +344,8 @@ void KSegView::mousePressEvent(QMouseEvent *e)
       return;
     }
 
-    if(tmp.count() == 0 || (e->state() & AltButton)) {
-      tmp = doc->whatLabelsAmIOn(e->x() + offsetX, e->y() + offsetY);
+    if(tmp.count() == 0 || ((e->state() & ControlButton) && (e->state() & ShiftButton))) {
+      tmp = doc->whatLabelsAmIOn(e->x(), e->y(), ViewTransform(offsetX, offsetY, zoom));
       
       if(tmp.count() == 0) {
 	if(!(e->state() & ShiftButton)) doc->clearSel();
@@ -316,7 +359,9 @@ void KSegView::mousePressEvent(QMouseEvent *e)
 
       drag = LABEL_DRAG;
       objectsDragged.append(tmp[0]);
-      dragOffset = tmp[0]->getLabelPos() - G_point(e->x() + offsetX, e->y() + offsetY);
+      objectsDragged[0]->getObject()->tempTransform(ViewTransform(offsetX, offsetY, zoom));
+      dragOffset = tmp[0]->getLabelPos() - G_point(e->x(), e->y());
+      objectsDragged[0]->getObject()->tempTransform(ViewTransform(-offsetX * zoom, -offsetY * zoom, 1. / zoom));
       return;
     }
 
@@ -332,7 +377,7 @@ void KSegView::mousePressEvent(QMouseEvent *e)
     //for a constrained point, grab the offset
     if(doc->getSelected().count() == 1 && doc->getSelected()[0]->getType() == G_POINT) {
       dragOffset = doc->getSelected()[0]->getObject()->getPoint() -
-	G_point(e->x() + offsetX, e->y() + offsetY);
+	(G_point(e->x(), e->y()) / zoom - G_point(offsetX, offsetY));
     }
 
     setCursor(pointingHandCursor);
@@ -341,7 +386,7 @@ void KSegView::mousePressEvent(QMouseEvent *e)
     else drag = NO_DRAG;
   }
   else if(e->button() == RightButton) { //-----------RIGHT BUTTON PRESSED
-    G_refs tmp = doc->whatAmIOn(e->x() + offsetX, e->y() + offsetY);
+    G_refs tmp = doc->whatAmIOn(e->x(), e->y(), ViewTransform(offsetX, offsetY, zoom));
 
     int i;
     if(tmp.count() > 2) return;
@@ -356,23 +401,27 @@ void KSegView::mousePressEvent(QMouseEvent *e)
     G_ref *p;
 
     if(tmp.count() == 2) { //create intersection point
-      if((G_point(e->x() + offsetX, e->y() + offsetY) -
-	  tmp[0]->getObject()->getCurveRef()->
-	  getIntersection(tmp[1]->getObject()->getCurveRef())).length() < 5) {
+      G_point int1Pt = tmp[0]->getObject()->getCurveRef()->
+	getIntersection(tmp[1]->getObject()->getCurveRef());
+      G_point int2Pt = tmp[0]->getObject()->getCurveRef()->
+	getIntersection(tmp[1]->getObject()->getCurveRef(), 1);
+
+      int1Pt = zoom * (int1Pt + G_point(offsetX, offsetY));
+      int2Pt = zoom * (int2Pt + G_point(offsetX, offsetY));
+
+      if((G_point(e->x(), e->y()) - int1Pt).length() < 5) {
 	if(!(e->state() & ShiftButton)) doc->clearSel();
 
-	p = new G_ref;
+	p = new G_ref();
 	p->create(G_POINT, G_INTERSECTION_POINT, tmp, doc);
 	p->update();
 	doc->addSel(p);
 
       }
-      else if((G_point(e->x() + offsetX, e->y() + offsetY) -
-	       tmp[0]->getObject()->getCurveRef()->
-	       getIntersection(tmp[1]->getObject()->getCurveRef(), 1)).length() < 5) {
+      else if((G_point(e->x(), e->y()) - int2Pt).length() < 5) {
 	if(!(e->state() & ShiftButton)) doc->clearSel();
 
-	p = new G_ref;
+	p = new G_ref();
 	p->create(G_POINT, G_INTERSECTION2_POINT, tmp, doc);
 	p->update();
 	doc->addSel(p);
@@ -384,9 +433,9 @@ void KSegView::mousePressEvent(QMouseEvent *e)
     if(tmp.count() == 1) { //create constrained point
       if(!(e->state() & ShiftButton)) doc->clearSel();
 
-      p = new G_ref;
+      p = new G_ref();
       p->create(G_POINT, G_CONSTRAINED_POINT, tmp, doc);
-      ((G_pointObject*)(p->getObject()))->setPoint(G_point(e->x() + offsetX, e->y() + offsetY));
+      ((G_pointObject*)(p->getObject()))->setPoint(G_point(e->x(), e->y()) / zoom - G_point(offsetX, offsetY));
       p->update();
       doc->addSel(p);
     }
@@ -394,9 +443,9 @@ void KSegView::mousePressEvent(QMouseEvent *e)
     if(tmp.count() == 0) { //create free point
       if(!(e->state() & ShiftButton)) doc->clearSel();
 	
-      p = new G_ref;
+      p = new G_ref();
       p->create(G_POINT, G_FREE_POINT, G_refs(), doc);
-      ((G_pointObject*)(p->getObject()))->setPoint(G_point(e->x() + offsetX, e->y() + offsetY));
+      ((G_pointObject*)(p->getObject()))->setPoint(G_point(e->x(), e->y()) / zoom - G_point(offsetX, offsetY));
       doc->addSel(p);
     }
   }
@@ -408,7 +457,9 @@ void KSegView::mousePressEvent(QMouseEvent *e)
 //--------------------------MOUSE MOVE EVENT-----------------------------
 void KSegView::mouseMoveEvent(QMouseEvent *e)
 {
-  if(drag == RECONSTRAIN_DRAG || drag == LABEL_DRAG || drag == MOVE_DRAG) {
+  if(doc == NULL) return;
+
+  if(drag != NO_DRAG) {
     KSegConstructionList::disableRedraws();
   }
 
@@ -426,7 +477,9 @@ void KSegView::mouseMoveEvent(QMouseEvent *e)
   }
 
   if(drag == LABEL_DRAG) {
-    objectsDragged[0]->setLabelPos(dragOffset + G_point(e->x() + offsetX, e->y() + offsetY));
+    objectsDragged[0]->getObject()->tempTransform(ViewTransform(offsetX, offsetY, zoom));
+    objectsDragged[0]->setLabelPos(dragOffset + G_point(e->x(), e->y()));
+    objectsDragged[0]->getObject()->tempTransform(ViewTransform(-offsetX * zoom, -offsetY * zoom, 1. / zoom));
     doc->emitDocumentChanged();
     doc->emitDocumentModified();
   }
@@ -439,7 +492,7 @@ void KSegView::mouseMoveEvent(QMouseEvent *e)
     int i;
 
     for(i = 0; i < (int)objectsDragged.count(); i++) {
-      objectsDragged[i]->getObject()->translate(G_point(e->x() - lastMouseX, e->y() - lastMouseY));
+      objectsDragged[i]->getObject()->translate(G_point(e->x() - lastMouseX, e->y() - lastMouseY) / zoom);
     }
     objectsUpdating.update();
     
@@ -462,7 +515,7 @@ void KSegView::mouseMoveEvent(QMouseEvent *e)
     if(objectsDragged.count() == 1 && objectsDragged[0]->getType() == G_POINT &&
        objectsDragged[0]->getDescendType() == G_CONSTRAINED_POINT) { // a constrained point is moving.
       ((G_pointObject *)(objectsDragged[0]->getObject()))->
-	setPoint(dragOffset + G_point(e->x() + offsetX, e->y() + offsetY));
+	setPoint(dragOffset + G_point(e->x(), e->y()) / zoom - G_point(offsetX, offsetY));
       
       objectsUpdating.update();
       lastMouseX = e->x(); lastMouseY = e->y();
@@ -474,7 +527,7 @@ void KSegView::mouseMoveEvent(QMouseEvent *e)
       hash_map<G_ref *, G_matrix>::iterator it;
       for(it = dragPattern.begin(); it != dragPattern.end(); ++it) {
 	(*it).first->getObject()->translate((*it).second *
-					    G_point(e->x() - lastMouseX, e->y() - lastMouseY));
+					    G_point(e->x() - lastMouseX, e->y() - lastMouseY) / zoom);
       }
 
       objectsUpdating.update();
@@ -484,12 +537,33 @@ void KSegView::mouseMoveEvent(QMouseEvent *e)
     }
   }
 
+  if(drag == PAN_DRAG) {
+    offsetX += ((double)(e->x() - lastMouseX)) / zoom;
+    offsetY += ((double)(e->y() - lastMouseY)) / zoom;
+
+    redrawBuffer();
+  }
+
+  if(drag == ZOOM_DRAG) {
+    double diff = e->y() - lastMouseY;
+    double scale = pow(2., -diff / 100.);
+
+    offsetX += (1. / scale - 1) * ((double)dragStartX) / zoom;
+    offsetY += (1. / scale - 1) * ((double)dragStartY) / zoom;
+    zoom *= scale;
+
+    redrawBuffer();
+    updateStatusBar(NoButton);
+  }
+
   lastMouseX = e->x(); lastMouseY = e->y();
 }
 
 //--------------------------MOUSE RELEASE EVENT-----------------------------
 void KSegView::mouseReleaseEvent(QMouseEvent *e)
 {
+  if(doc == NULL) return;
+
   KSegConstructionList::enableRedraws();
 
   if(drag == NO_DRAG) return;
@@ -499,8 +573,8 @@ void KSegView::mouseReleaseEvent(QMouseEvent *e)
   if(drag == MOVE_DRAG && doc->isConstruction()) doc->emitDocumentChanged();
 
   if(drag == RECTANGLE_DRAG) {
-    doc->addSelect(QRect(dragStartX + offsetX, dragStartY + offsetY,
-			 dragCurX - dragStartX, dragCurY - dragStartY).normalize());
+    doc->addSelect(QRect(dragStartX, dragStartY, dragCurX - dragStartX,
+			 dragCurY - dragStartY).normalize(), ViewTransform(offsetX, offsetY, zoom));
 
     drag = NO_DRAG;
     doc->emitDocumentChanged();
@@ -511,9 +585,10 @@ void KSegView::mouseReleaseEvent(QMouseEvent *e)
       objectsUpdating.topologicalSort(objectsDragged);
     }
 
-    G_point releasePt = objectsDragged[0]->getObject()->getPoint();
+    G_point releasePt = zoom * (objectsDragged[0]->getObject()->getPoint() + G_point(offsetX, offsetY));
 
-    G_refs tmp = doc->whatAmIOn(ROUND(releasePt.getX()), ROUND(releasePt.getY()), false);
+    G_refs tmp = doc->whatAmIOn(ROUND(releasePt.getX()), ROUND(releasePt.getY()),
+				ViewTransform(offsetX, offsetY, zoom), false);
 
     int i;
 
@@ -567,22 +642,23 @@ void KSegView::mouseReleaseEvent(QMouseEvent *e)
 
     if(tmp.count() == 2 && tmp[0]->getType() & G_CURVE && tmp[1]->getType() & G_CURVE) {
       //make it an intersection point
-      if((G_point(releasePt.getX(), releasePt.getY()) -
-	  tmp[0]->getObject()->getCurveRef()->
-	  getIntersection(tmp[1]->getObject()->getCurveRef())).length() < 5) {
+      G_point int1Pt = tmp[0]->getObject()->getCurveRef()->
+	getIntersection(tmp[1]->getObject()->getCurveRef());
+      G_point int2Pt = tmp[0]->getObject()->getCurveRef()->
+	getIntersection(tmp[1]->getObject()->getCurveRef(), 1);
 
+      int1Pt = zoom * (int1Pt + G_point(offsetX, offsetY));
+      int2Pt = zoom * (int2Pt + G_point(offsetX, offsetY));
+
+      if((releasePt - int1Pt).length() < 5) {
 	objectsDragged[0]->reconstrain(G_INTERSECTION_POINT, tmp);
 	objectsUpdating.update();
 	doc->emitDocumentChanged();
       }
-      else if((G_point(releasePt.getX(), releasePt.getY()) -
-	       tmp[0]->getObject()->getCurveRef()->
-	       getIntersection(tmp[1]->getObject()->getCurveRef(), 1)).length() < 5) {
-
+      else if((releasePt - int2Pt).length() < 5) {
 	objectsDragged[0]->reconstrain(G_INTERSECTION2_POINT, tmp);
 	objectsUpdating.update();
 	doc->emitDocumentChanged();
-
       }
     }
 
@@ -599,6 +675,9 @@ void KSegView::mouseReleaseEvent(QMouseEvent *e)
 
   }
 
+  if(drag == ZOOM_DRAG) nextZoom = false;
+  if(drag == PAN_DRAG) nextPan = false;
+
   drag = NO_DRAG;
   objectsDragged.clear();
   objectsUpdating.clear();
@@ -609,11 +688,13 @@ void KSegView::mouseReleaseEvent(QMouseEvent *e)
 //-------------------------MOUSE DOUBLE CLICK EVENT-----------------------------
 void KSegView::mouseDoubleClickEvent(QMouseEvent *e)
 {
-  if(e->button() == LeftButton) {
-    G_refs tmp = doc->whatAmIOn(e->x() + offsetX, e->y() + offsetY);
+  if(doc == NULL) return;
 
-    if(tmp.count() == 0 || (e->state() & AltButton)) {
-      tmp = doc->whatLabelsAmIOn(e->x() + offsetX, e->y() + offsetY);
+  if(e->button() == LeftButton) {
+    G_refs tmp = doc->whatAmIOn(e->x(), e->y(), ViewTransform(offsetX, offsetY, zoom));
+
+    if(tmp.count() == 0 || ((e->state() & ControlButton) && (e->state() & ShiftButton))) {
+      tmp = doc->whatLabelsAmIOn(e->x(), e->y(), ViewTransform(offsetX, offsetY, zoom));
 
       if(tmp.count() == 0) return;
 
@@ -660,11 +741,26 @@ void KSegView::updateStatusBar(ButtonState state)
 {
   QPoint e = mapFromGlobal(QCursor::pos());
 
-  if(drag == NO_DRAG) {
-    G_refs tmp = doc->whatAmIOn(e.x() + offsetX, e.y() + offsetY);
+  if(nextPan) {
+    emit statusBarMessage(tr("Pan"));
+    setCursor(SizeAllCursor);
+    return;
+  }
 
-    if(tmp.count() == 0) {
-      tmp = doc->whatLabelsAmIOn(e.x() + offsetX, e.y() + offsetY);
+  if(nextZoom) {
+    emit statusBarMessage(tr("Zoom"));
+    QPixmap p((const char **)zoom_cursor_xpm);
+    QBitmap b;
+    b = p;
+    setCursor(QCursor(b, b, 9, 9));
+    return;
+  }
+
+  if(drag == NO_DRAG) {
+    G_refs tmp = doc->whatAmIOn(e.x(), e.y(), ViewTransform(offsetX, offsetY, zoom));
+
+    if(tmp.count() == 0 || ((state & ControlButton) && (state & ShiftButton))) {
+      tmp = doc->whatLabelsAmIOn(e.x(), e.y(), ViewTransform(offsetX, offsetY, zoom));
 
       if(tmp.count() == 0) {
 	unsetCursor();
@@ -731,9 +827,10 @@ void KSegView::updateStatusBar(ButtonState state)
     messageTwo = messageTwo.arg(KFormula::toUgly(objectsDragged[0]->getLabel().getText()));
 
     
-    G_point releasePt = objectsDragged[0]->getObject()->getPoint();
+    G_point releasePt = zoom * (objectsDragged[0]->getObject()->getPoint() + G_point(offsetX, offsetY));
 
-    G_refs tmp = doc->whatAmIOn(ROUND(releasePt.getX()), ROUND(releasePt.getY()), false);
+    G_refs tmp = doc->whatAmIOn(ROUND(releasePt.getX()), ROUND(releasePt.getY()),
+				ViewTransform(offsetX, offsetY, zoom), false);
 
     int i;
 
@@ -758,15 +855,16 @@ void KSegView::updateStatusBar(ButtonState state)
       message = message.arg(KFormula::toUgly(tmp[0]->getLabel().getText()));
     }
     else if(tmp.count() == 2 && tmp[0]->getType() & G_CURVE && tmp[1]->getType() & G_CURVE) {
-      //that's a long if statement--checks if it's really near the intersection or just
-      //near both curves
-      if((G_point(releasePt.getX(), releasePt.getY()) -
-	  tmp[0]->getObject()->getCurveRef()->
-	  getIntersection(tmp[1]->getObject()->getCurveRef())).length() < 5 ||
-	 (G_point(releasePt.getX(), releasePt.getY()) -
-	  tmp[0]->getObject()->getCurveRef()->
-	  getIntersection(tmp[1]->getObject()->getCurveRef(), 1)).length() < 5) {
+      //check if it's really near the intersection or just near both curves
+      G_point int1Pt = tmp[0]->getObject()->getCurveRef()->
+	getIntersection(tmp[1]->getObject()->getCurveRef());
+      G_point int2Pt = tmp[0]->getObject()->getCurveRef()->
+	getIntersection(tmp[1]->getObject()->getCurveRef(), 1);
 
+      int1Pt = zoom * (int1Pt + G_point(offsetX, offsetY));
+      int2Pt = zoom * (int2Pt + G_point(offsetX, offsetY));
+
+      if((releasePt - int1Pt).length() < 5 || (releasePt - int2Pt).length() < 5) {
 	message = messageTwo;
 	message = message.arg(G_ref::getNameFromType(tmp[0]->getType()));
 	message = message.arg(KFormula::toUgly(tmp[0]->getLabel().getText()));
@@ -789,23 +887,117 @@ void KSegView::playConstruction(int which)
   QPainter p;
 
   constructionPainter = &p;
+  constructionTransform = ViewTransform(offsetX, offsetY, zoom);
 
   p.begin(this);
   p.setClipRect(buffer.rect());
-  QRect window = p.viewport();
-  window.moveBy(offsetX, offsetY);
-  p.setWindow(window);
   
   doc->doPlay(which);
 
   p.end();
 }
 
+void KSegView::updateAllViews()
+{
+  int i;
+
+  if(KSegProperties::getProperty("SelectType").lower() == "borderselect")
+    selectType = BORDER_SELECT;
+  else if(KSegProperties::getProperty("SelectType").lower() == "blinkingselect")
+    selectType = BLINKING_SELECT;
+  
+  for(i = 0; i < (int)allViews.count(); ++i) {
+    allViews.at(i)->redrawBuffer();
+  }
+}
+
+void KSegView::viewPan()
+{
+  nextPan = !nextPan;
+  if(nextZoom) nextZoom = false;
+
+  updateStatusBar(NoButton);
+  emit updateMenus();
+}
+
+void KSegView::viewZoom()
+{
+  nextZoom = !nextZoom;
+  if(nextPan) nextPan = false;
+
+  updateStatusBar(NoButton);
+  emit updateMenus();
+}
+
+void KSegView::viewZoomFit()
+{
+  QRect tmp = doc->getSize(ViewTransform(0, 0, 1));
+
+  double oldOffsetX = offsetX, oldOffsetY = offsetY, oldZoom = zoom;
+
+  if(tmp.isEmpty() || tmp.isNull()) return;
+
+  QRect r = rect();
+  r.setWidth(r.width() - ScrollBarThickness);
+  r.setHeight(r.height() - ScrollBarThickness);
+  
+  zoom = QMIN(((double)r.width()) / (1.05 * (double)tmp.width()),
+	      ((double)r.height()) / (1.05 * (double)tmp.height()));
+
+  QPoint rc = r.center();
+  QPoint tc = tmp.center();
+
+  offsetX = ((double)rc.x()) / zoom - tc.x();
+  offsetY = ((double)rc.y()) / zoom - tc.y();
+
+  zoomAnimation(oldOffsetX, oldOffsetY, oldZoom);
+  updateStatusBar(NoButton);
+}
+
+
+void KSegView::viewOrigZoom()
+{
+  double oldOffsetX = offsetX, oldOffsetY = offsetY, oldZoom = zoom;
+
+  offsetX = 0;
+  offsetY = 0;
+  zoom = 1;
+
+  zoomAnimation(oldOffsetX, oldOffsetY, oldZoom);
+  updateStatusBar(NoButton);
+}
+
+void KSegView::zoomAnimation(double oldOffsetX, double oldOffsetY, double oldZoom)
+{
+  QTime t;
+  t.start();
+
+  double newOffsetX = offsetX, newOffsetY = offsetY, newZoom = zoom;
+
+  int zoomTime = 700;
+  
+  DragType oldDrag = drag;
+  drag = ZOOM_DRAG; //to prevent menus from updating
+
+  if(fabs(offsetX - oldOffsetX) + fabs(offsetY - oldOffsetY) + fabs(zoom - oldZoom) > 0.0001)
+  while(t.elapsed() < zoomTime) {
+    double p = ((double)t.elapsed()) / ((double)zoomTime);
+
+    zoom = exp(p * log(newZoom) + (1 - p) * log(oldZoom));
+    if(fabs(oldZoom - newZoom) > 0.000001) p = (zoom - oldZoom) / (newZoom - oldZoom);
+    offsetX = ((1 - p) * oldOffsetX * oldZoom + p * (newOffsetX * newZoom)) / zoom;
+    offsetY = ((1 - p) * oldOffsetY * oldZoom + p * (newOffsetY * newZoom)) / zoom;
+    redrawBuffer();
+    QApplication::syncX();
+  }
+
+  offsetX = newOffsetX;
+  offsetY = newOffsetY;
+  zoom = newZoom;
+  redrawBuffer();
+
+  drag = oldDrag;
+}
+
 #include "KSegView.moc"
 
-
-
-
-
-
-
