@@ -25,6 +25,7 @@
 #include "kformula.H"
 #include <stdio.h>
 #include <ctype.h>
+#include <iostream>
 
 template class QList<box>;
 
@@ -171,8 +172,9 @@ void box::calculate(QPainter &p, int setFontsize, QFont *f_, QColor *, QColor *)
   lastFont = f;
   lastFont.setPointSize(fontsize);
   p.setFont(lastFont);
-  QFontInfo font_info_tmp(lastFont);
-  fontsize = font_info_tmp.pointSize();
+  //QFontInfo font_info_tmp(lastFont);
+  //QFontInfo font_info_tmp = p.fontInfo();
+  //fontsize = font_info_tmp.pointSize();
 
   QFontMetrics fm = p.fontMetrics();
   if ( f_ )
@@ -223,6 +225,14 @@ void box::calculate(QPainter &p, int setFontsize, QFont *f_, QColor *, QColor *)
       else {
 	rect = fm.boundingRect(text);
       }
+
+      
+      { // fix the height of the box to not depend on the letters
+	QRect heightRect = fm.boundingRect("X");
+	rect.setTop(heightRect.top());
+	rect.setBottom(heightRect.bottom());
+      }
+
       rect.moveBy(0, -fm.boundingRect("+").center().y());
       break;
 
