@@ -1,5 +1,5 @@
 <!DOCTYPE TS><TS>
-<codec>iso8859-1</codec>
+<defaultcodec>iso8859-1</defaultcodec>
 <context>
     <name>EditorToolBar</name>
     <message>
@@ -172,7 +172,7 @@
     </message>
     <message>
         <source>Insert %1 %2</source>
-        <comment>For example, "Insert Measurement M1"</comment>
+        <comment>For example, &quot;Insert Measurement M1&quot;</comment>
         <translation>Vstavit&apos; %1 %2</translation>
     </message>
     <message>
@@ -346,6 +346,18 @@
         <source>%1 is the radius of %2</source>
         <translation>%1 eto radius %2</translation>
     </message>
+    <message>
+        <source>%1 is the endpoint of %2</source>
+        <translation>%1 konechnaya tochka %2</translation>
+    </message>
+    <message>
+        <source>%1 is the 2nd endpoint of %2</source>
+        <translation>%1 vtoraya konechnaya tochka %2</translation>
+    </message>
+    <message>
+        <source>%1 is the center of %2</source>
+        <translation>%1 eto centr %2</translation>
+    </message>
 </context>
 <context>
     <name>KSegDocument</name>
@@ -355,14 +367,42 @@
     </message>
     <message>
         <source>OK</source>
-        <translation type='unfinished'>OK</translation>
+        <translation>OK</translation>
     </message>
 </context>
 <context>
-    <name>KSegDocumnent</name>
+    <name>KSegPreferencesDialog</name>
+    <message>
+        <source>KSEG Preferences</source>
+        <translation>KSEGovskie Predpochteniya</translation>
+    </message>
+    <message>
+        <source>Background Color</source>
+        <translation>Cvet Fona</translation>
+    </message>
+    <message>
+        <source>Default Foreground Color</source>
+        <translation>Nachal&apos;nyj Cvet Objektov</translation>
+    </message>
+    <message>
+        <source>Selection Display</source>
+        <translation>Kak Risovat&apos; Vybrannye Objekty</translation>
+    </message>
+    <message>
+        <source>Border Selection</source>
+        <translation>Risovat&apos; Bordyurom</translation>
+    </message>
+    <message>
+        <source>Blinking Selection</source>
+        <translation>Risovat&apos; Miganiem</translation>
+    </message>
     <message>
         <source>OK</source>
-        <translation type='obsolete'>OK</translation>
+        <translation>OK</translation>
+    </message>
+    <message>
+        <source>Cancel</source>
+        <translation>Otmena</translation>
     </message>
 </context>
 <context>
@@ -403,6 +443,105 @@
         <translation>Otmena</translation>
     </message>
 </context>
+<context>
+    <name>KSegSelectionGroupDialog</name>
+    <message>
+        <source>Select</source>
+        <translation>Vybrat&apos;</translation>
+    </message>
+    <message>
+        <source>Deselect</source>
+        <translation>Ubrat&apos; Vybor</translation>
+    </message>
+    <message>
+        <source>New Group</source>
+        <translation>Novaya Gruppa</translation>
+    </message>
+    <message>
+        <source>Delete Group</source>
+        <translation>Udalit&apos; Gruppu</translation>
+    </message>
+    <message>
+        <source>Close</source>
+        <translation>Zakryt&apos;</translation>
+    </message>
+    <message>
+        <source>Affect
+Invisible
+Items</source>
+        <translation>Trogat&apos;
+Nevidimye
+Objekty</translation>
+    </message>
+    <message>
+        <source>Hold SHIFT and click to add to current selection</source>
+        <translation>Derzhite SHIFT i clicknite chto-b dobavit&apos; k vybory</translation>
+    </message>
+    <message>
+        <source>Whether the &quot;All &lt;Type&gt;&quot; groups also (de)select invisible objects</source>
+        <translation>Vybirayut-li gruppy tipa &quot;Vse &lt;Tip&gt;&quot; nevidimye objekty</translation>
+    </message>
+    <message>
+        <source>All Points</source>
+        <translation>Vse tochki</translation>
+    </message>
+    <message>
+        <source>All Segments</source>
+        <translation>Vse Otrezki</translation>
+    </message>
+    <message>
+        <source>All Rays</source>
+        <translation>Vse Luchi</translation>
+    </message>
+    <message>
+        <source>All Lines</source>
+        <translation>Vse Linii</translation>
+    </message>
+    <message>
+        <source>All Circles</source>
+        <translation>Vse Krugi</translation>
+    </message>
+    <message>
+        <source>All Arcs</source>
+        <translation>Vse Dugi</translation>
+    </message>
+    <message>
+        <source>All Polygons</source>
+        <translation>Vse Mnogougol&apos;niki</translation>
+    </message>
+    <message>
+        <source>All Circle Interiors</source>
+        <translation>Vse Vnutrennosti Krugov</translation>
+    </message>
+    <message>
+        <source>All Arc Sectors</source>
+        <translation>Vse Sektory</translation>
+    </message>
+    <message>
+        <source>All Arc Segments</source>
+        <translation>Vse Segmenty</translation>
+    </message>
+    <message>
+        <source>All Loci</source>
+        <translation>Vse Puchki</translation>
+    </message>
+    <message>
+        <source>All Measurements</source>
+        <translation>Vse Izmereniya</translation>
+    </message>
+    <message>
+        <source>All Calculations</source>
+        <translation>Vse Vychisleniya</translation>
+    </message>
+    <message>
+        <source>Enter new group name:</source>
+        <translation>Vvediti nazvanie novoy gruppy:</translation>
+    </message>
+    <message>
+        <source>Are you sure you want to delete the selected group(s)?</source>
+        <translation>Tochno hochesh udalit&apos; vybrannye gruppy?</translation>
+    </message>
+</context>
 <context>
     <name>KSegView</name>
     <message>
@@ -415,7 +554,7 @@
     </message>
     <message>
         <source>Move label of %1 %2</source>
-        <comment>E.g. "Move label of Point P1"</comment>
+        <comment>E.g. &quot;Move label of Point P1&quot;</comment>
         <translation>Podvinut&apos; nazvaniye %1 %2</translation>
     </message>
     <message>
@@ -440,13 +579,21 @@
     </message>
     <message>
         <source>Reconstrain %1 %2 to %3 %4</source>
-        <comment>E.g., "Reconstrain Point P1 to Line l1"</comment>
+        <comment>E.g., &quot;Reconstrain Point P1 to Line l1&quot;</comment>
         <translation>Pereopredelit&apos; %1 %2 chto-by lezhala na %3 %4</translation>
     </message>
     <message>
         <source>Reconstrain %1 %2 to the intersection of %3 %4 and %5 %6</source>
         <translation>Pereopredelit&apos; %1 %2 chto-b lezhala na peresechenii %3 %4 i %5 %6</translation>
     </message>
+    <message>
+        <source>Pan</source>
+        <translation>Dviganie Vida</translation>
+    </message>
+    <message>
+        <source>Zoom</source>
+        <translation>Uvelichenie</translation>
+    </message>
 </context>
 <context>
     <name>KSegWindow</name>
@@ -532,7 +679,7 @@
     </message>
     <message>
         <source>&amp;Delete Objects</source>
-        <translation>U&amp;dalit&apos; Objekt</translation>
+        <translation>U&amp;dalit&apos; Objekty</translation>
     </message>
     <message>
         <source>Toggle &amp;Label</source>
@@ -834,16 +981,6 @@
         <source>About KSEG</source>
         <translation>O KSEGe</translation>
     </message>
-    <message>
-        <source>KSEG v0.3
-Copyright (C) 1999-2001 Ilya Baran
-http://www.mit.edu/~ibaran
-ibaran@mit.edu</source>
-        <translation>KSEG v0.3
-Copyright (C) 1999-2001 Ilya Baran
-http://www.mit.edu/~ibaran
-ibaran@mit.edu</translation>
-    </message>
     <message>
         <source>&amp;Change Directory...</source>
         <translation>Izmenit&apos; &amp;Direktoriyu...</translation>
@@ -860,6 +997,96 @@ ibaran@mit.edu</translation>
         <source>You will need to restart KSEG for the language change to fully take effect.</source>
         <translation>Vy dolzhny perezapustit&apos; KSEG chto-by izmeneniye yazyka voshlo v silu.</translation>
     </message>
+    <message>
+        <source>&amp;Delete Object</source>
+        <translation>U&amp;dalit&apos; Objekt</translation>
+    </message>
+    <message>
+        <source>&amp;Hide Object</source>
+        <translation>Spryac&amp;h Objekt</translation>
+    </message>
+    <message>
+        <source>Pr&amp;eferences...</source>
+        <translation>P&amp;redpochteniya...</translation>
+    </message>
+    <message>
+        <source>KSEG v0.4
+Copyright (C) 1999-2003 Ilya Baran
+http://www.mit.edu/~ibaran
+ibaran@mit.edu</source>
+        <translation>KSEG v0.4
+Copyright (C) 1999-2003 Ilya Baran
+http://www.mit.edu/~ibaran
+ibaran@mit.edu</translation>
+    </message>
+    <message>
+        <source>Open In Current Window...</source>
+        <translation>Otkryt&apos; V Etom Okne...</translation>
+    </message>
+    <message>
+        <source>Copy As Construction</source>
+        <translation>Skopirovat&apos; Kak Postroenie</translation>
+    </message>
+    <message>
+        <source>Selection &amp;Groups...</source>
+        <translation>&amp;Gruppy Vybora...</translation>
+    </message>
+    <message>
+        <source>&amp;Zoom</source>
+        <translation>Uvelichenie (&amp;Z)</translation>
+    </message>
+    <message>
+        <source>&amp;Pan</source>
+        <translation>Dviganie Vida (&amp;P)</translation>
+    </message>
+    <message>
+        <source>&amp;View</source>
+        <translation>&amp;Vid</translation>
+    </message>
+    <message>
+        <source>Zoom To &amp;Fit</source>
+        <translation>Uvelichenie Do Upora (&amp;F)</translation>
+    </message>
+    <message>
+        <source>&amp;Original Zoom (100%)</source>
+        <translation>Iznachal&apos;n&amp;oe Uvelichenie</translation>
+    </message>
+    <message>
+        <source>E&amp;xport To Image...</source>
+        <translation>E&amp;xportirovat&apos; kak kartinku...</translation>
+    </message>
+    <message>
+        <source>Endpoints</source>
+        <translation>Konechnye Tochki</translation>
+    </message>
+    <message>
+        <source>Center Point</source>
+        <translation>Tochka V Centre</translation>
+    </message>
+    <message>
+        <source>Endpoint</source>
+        <translation>Konechnaya Tochka</translation>
+    </message>
+    <message>
+        <source>Zoom:</source>
+        <translation>Uvelichenie:</translation>
+    </message>
+    <message>
+        <source>Choose the File Format:</source>
+        <translation>Vyberete Format Fayla:</translation>
+    </message>
+    <message>
+        <source>High Quality Image</source>
+        <translation>Kartinka Vysokogo Kachestva</translation>
+    </message>
+    <message>
+        <source>OK</source>
+        <translation>OK</translation>
+    </message>
+    <message>
+        <source>Exporting, Please wait...</source>
+        <translation>Terpi, Ya Exportiruyu...</translation>
+    </message>
 </context>
 <context>
     <name>MatrixDialog</name>
@@ -884,14 +1111,4 @@ ibaran@mit.edu</translation>
         <translation>Otmena</translation>
     </message>
 </context>
-<context>
-    <name>Reverse Dragging Error</name>
-    <message>
-        <source>Critical Error in BuildReverse algorithm.
-Please send the file of this drawing to the author as a bug report.
-</source>
-        <translation>Proizoshla kriticheskaya osibka v algoritme reverse dragging."n
-Pozhaluysta, poshlite eto postroyeniye avtoru KSEGa kak bug report."n</translation>
-    </message>
-</context>
 </TS>
