@@ -1,6 +1,6 @@
 /*
  *  KSeg
- *  Copyright (C) 1999-2001 Ilya Baran
+ *  Copyright (C) 1999-2003 Ilya Baran
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,13 +20,7 @@
  *                 ibaran@mit.edu
  */
 
-
-#include "KSegDocument.H"
-#include "G_ref.H"
-#include "G_object.H"
-
-#include "formula/kformulaedit.H"
-
+#include <iostream>
 #include <qlayout.h>
 #include <qdialog.h>
 #include <qpushbutton.h>
@@ -36,6 +30,13 @@
 #include "KSegConstruction.H"
 #include "KSegRecursionDialog.H"
 #include "G_refSearcher.H"
+#include "KSegDocument.H"
+#include "G_ref.H"
+#include "G_object.H"
+#include "KSegSelectionGroupDialog.H"
+
+#include "formula/kformulaedit.H"
+
 
 //this file contains all the menu handlers by KSegDocument.
 
@@ -52,7 +53,7 @@ void KSegDocument::newSegment()
   }
 
   if(selectedRefs.count() == 2) { // two points
-    G_ref *s = new G_ref;
+    G_ref *s = new G_ref();
     s->create(G_SEGMENT, G_ENDPOINTS_SEGMENT, selectedRefs, this);
     clearSel();
     addSel(s);
@@ -61,7 +62,7 @@ void KSegDocument::newSegment()
   }
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *s = new G_ref;
+    G_ref *s = new G_ref();
     tmp.clear();
     tmp.append(selectedRefs[i]);
     if(i + 1 < (int)selectedRefs.count()) {
@@ -92,7 +93,7 @@ void KSegDocument::newRay()
   }
 
   if(selectedRefs.count() == 2) { // two points
-    G_ref *s = new G_ref;
+    G_ref *s = new G_ref();
     s->create(G_RAY, G_TWOPOINTS_RAY, selectedRefs, this);
     clearSel();
     addSel(s);
@@ -101,7 +102,7 @@ void KSegDocument::newRay()
   }
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *s = new G_ref;
+    G_ref *s = new G_ref();
     tmp.clear();
     tmp.append(selectedRefs[i]);
     if(i + 1 < (int)selectedRefs.count()) {
@@ -127,7 +128,7 @@ void KSegDocument::newBisector()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_RAY, G_BISECTOR_RAY, selectedRefs, this);
 
@@ -146,12 +147,12 @@ void KSegDocument::newIntersection()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_POINT, G_INTERSECTION_POINT, selectedRefs, this);
 
   if(!(selectedRefs[0]->getType() & G_STRAIGHT && selectedRefs[1]->getType() & G_STRAIGHT)) {
-    G_ref *p1 = new G_ref;
+    G_ref *p1 = new G_ref();
 
     p1->create(G_POINT, G_INTERSECTION2_POINT, selectedRefs, this);
     
@@ -170,6 +171,56 @@ void KSegDocument::newIntersection()
 }
 
 
+void KSegDocument::newEndpoints()
+{
+  if(!canNewEndpoints()) {
+    ASSERT(canNewEndpoints());
+    return;
+  }
+
+  G_ref *p = new G_ref();
+
+  p->create(G_POINT, G_END_POINT, selectedRefs, this);
+
+  if(!(selectedRefs[0]->getType() & G_RAY)) {
+    G_ref *p1 = new G_ref();
+
+    p1->create(G_POINT, G_END2_POINT, selectedRefs, this);
+    
+    clearSel();
+
+    addSel(p);
+    addSel(p1);
+  }
+  else {
+    clearSel();
+
+    addSel(p);
+  }
+
+  emit documentChanged();
+}
+
+
+void KSegDocument::newCenterpoint()
+{
+  if(!canNewCenterpoint()) {
+    ASSERT(canNewCenterpoint());
+    return;
+  }
+
+  G_ref *p = new G_ref();
+
+  p->create(G_POINT, G_CENTER_POINT, selectedRefs, this);
+
+  clearSel();
+
+  addSel(p);
+
+  emit documentChanged();
+}
+
+
 void KSegDocument::newCircle()
 {
   if(!canNewCircle()) {
@@ -179,7 +230,7 @@ void KSegDocument::newCircle()
 
   G_refs tmp;
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   if(selectedRefs[0]->getType() == G_POINT && selectedRefs[1]->getType() == G_POINT) {
     //by center and point
@@ -213,7 +264,7 @@ void KSegDocument::newArc()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_ARC, G_THREEPOINTS_ARC, selectedRefs, this);
 
@@ -232,7 +283,7 @@ void KSegDocument::newArcSector()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_ARCSECTOR, G_DEFAULT_FILLED, selectedRefs, this);
 
@@ -251,7 +302,7 @@ void KSegDocument::newArcSegment()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_ARCSEGMENT, G_DEFAULT_FILLED, selectedRefs, this);
 
@@ -270,7 +321,7 @@ void KSegDocument::newCircleInterior()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_CIRCLEINTERIOR, G_DEFAULT_FILLED, selectedRefs, this);
 
@@ -289,7 +340,7 @@ void KSegDocument::newPolygon()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_POLYGON, G_DEFAULT_FILLED, selectedRefs, this);
 
@@ -308,7 +359,7 @@ void KSegDocument::newLocus()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   DescendantSearcher ds(selectedRefs[0]);
 
@@ -346,7 +397,7 @@ void KSegDocument::newMidpoint()
   }
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *p = new G_ref;
+    G_ref *p = new G_ref();
     tmp.clear();
     tmp.append(selectedRefs[i]);
 
@@ -381,7 +432,7 @@ void KSegDocument::newPerpendicular()
 
   if(pts.count() == 1) {
     for(i = 0; i < (int)straights.count(); i++) {
-      G_ref *p = new G_ref;
+      G_ref *p = new G_ref();
       tmp.clear();
       tmp.append(pts[0]);
       tmp.append(straights[i]);
@@ -393,7 +444,7 @@ void KSegDocument::newPerpendicular()
   }
   else { // straights.count() must be 1.
     for(i = 0; i < (int)pts.count(); i++) {
-      G_ref *p = new G_ref;
+      G_ref *p = new G_ref();
       tmp.clear();
       tmp.append(pts[i]);
       tmp.append(straights[0]);
@@ -431,7 +482,7 @@ void KSegDocument::newLine()
 
   if(pts.count() == 1) {
     for(i = 0; i < (int)straights.count(); i++) {
-      G_ref *p = new G_ref;
+      G_ref *p = new G_ref();
       tmp.clear();
       tmp.append(pts[0]);
       tmp.append(straights[i]);
@@ -443,7 +494,7 @@ void KSegDocument::newLine()
   }
   else if(straights.count() == 0) {
     if(selectedRefs.count() == 2) { // two points
-      G_ref *s = new G_ref;
+      G_ref *s = new G_ref();
       s->create(G_LINE, G_TWOPOINTS_LINE, selectedRefs, this);
       clearSel();
       addSel(s);
@@ -452,7 +503,7 @@ void KSegDocument::newLine()
     }
 
     for(i = 0; i < (int)selectedRefs.count(); i++) {
-      G_ref *s = new G_ref;
+      G_ref *s = new G_ref();
       tmp.clear();
       tmp.append(selectedRefs[i]);
       if(i + 1 < (int)selectedRefs.count()) {
@@ -467,7 +518,7 @@ void KSegDocument::newLine()
   }
   else { // straights.count() must be 1.
     for(i = 0; i < (int)pts.count(); i++) {
-      G_ref *p = new G_ref;
+      G_ref *p = new G_ref();
       tmp.clear();
       tmp.append(pts[i]);
       tmp.append(straights[0]);
@@ -498,8 +549,24 @@ void KSegDocument::editDelete()
 
   toBeDeleted.topologicalSort(selectedRefs);
 
+  //if this is a construction and we are deleting any givens, we must
+  //also delete all loops:
   int i;
 
+  if(isConstruction()) {
+    for(i = toBeDeleted.count() - 1; i >= 0; i--) {
+      if(toBeDeleted[i]->getGiven()) break;
+    }
+    
+    if(i >= 0) { //if there was a given
+      KSegConstruction *me = (KSegConstruction *)this;
+
+      for(i = me->getLoops().count() - 1; i >= 0; i--) {
+	toBeDeleted.appendUnique(me->getLoops()[i]);
+      }
+    }
+  }
+
   allRefs.holdRemovals();
 
   for(i = toBeDeleted.count() - 1; i >= 0; i--) {
@@ -745,7 +812,7 @@ void KSegDocument::measureDistance()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
   G_refs tmp;
 
   if(selectedRefs[0]->getType() == G_POINT) tmp = selectedRefs;
@@ -756,8 +823,9 @@ void KSegDocument::measureDistance()
 
   p->create(G_MEASURE, G_DISTANCE_MEASURE, tmp, this);
 
-  //move it close to the (first) point
-  p->getObject()->translate(selectedRefs[0]->getObject()->getPoint() +
+  //move it close to the (first) object
+  p->getObject()->translate(G_point(selectedRefs[0]->getObject()->getScrollExtents().
+				    center()) +
 			    G_point(INTRAND(-100, 100), INTRAND(-100, 100)));
 
   clearSel();
@@ -775,7 +843,7 @@ void KSegDocument::measureLength()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_MEASURE, G_LENGTH_MEASURE, selectedRefs, this);
 
@@ -799,7 +867,7 @@ void KSegDocument::measureRadius()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_MEASURE, G_RADIUS_MEASURE, selectedRefs, this);
 
@@ -822,7 +890,7 @@ void KSegDocument::measureAngle()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_MEASURE, G_ANGLE_MEASURE, selectedRefs, this);
 
@@ -846,7 +914,7 @@ void KSegDocument::measureRatio()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_MEASURE, G_RATIO_MEASURE, selectedRefs, this);
 
@@ -870,7 +938,7 @@ void KSegDocument::measureSlope()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_MEASURE, G_SLOPE_MEASURE, selectedRefs, this);
 
@@ -901,7 +969,7 @@ void KSegDocument::measureArea()
     return;
   }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_MEASURE, G_AREA_MEASURE, selectedRefs, this);
 
@@ -1007,7 +1075,7 @@ void KSegDocument::transformTranslate()
   int i;
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *p = new G_ref;
+    G_ref *p = new G_ref();
     parents = t_vector;
     parents.insert(0, selectedRefs[i]);
 
@@ -1035,7 +1103,7 @@ void KSegDocument::transformReflect()
   int i;
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *p = new G_ref;
+    G_ref *p = new G_ref();
     parents = t_mirror;
     parents.insert(0, selectedRefs[i]);
 
@@ -1063,7 +1131,7 @@ void KSegDocument::transformRotate()
   int i;
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *p = new G_ref;
+    G_ref *p = new G_ref();
     parents = t_angle;
     parents.insert(0, t_center[0]);
     parents.insert(0, selectedRefs[i]);
@@ -1092,7 +1160,7 @@ void KSegDocument::transformScale()
   int i;
 
   for(i = 0; i < (int)selectedRefs.count(); i++) {
-    G_ref *p = new G_ref;
+    G_ref *p = new G_ref();
     parents = t_ratio;
     parents.insert(0, t_center[0]);
     parents.insert(0, selectedRefs[i]);
@@ -1135,7 +1203,7 @@ void KSegDocument::measureCalculate()
     }
   }
 
-  assert(curView != 0);
+  ASSERT(curView != 0);
 
   QString str;
   if(selectedRefs.size() != 0) str = QString(QChar(int(REFERENCE_NUM(0))));
@@ -1145,7 +1213,7 @@ void KSegDocument::measureCalculate()
   e->run();
   if(e->result() == QDialog::Rejected) { delete e; return; }
 
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   p->create(G_CALCULATE, G_REGULAR_CALCULATE, e->getOutputParents(), this);
   ((G_calculateObject *)(p->getObject()))->setInitialFormulaString(e->getOutputString());
@@ -1162,6 +1230,41 @@ void KSegDocument::measureCalculate()
   return;
 }
 
+void KSegDocument::editShowSelectionGroupDialog()
+{
+  KSegView *curView = 0;
+  QList<KSegView> allViews = KSegView::getAllViews();
+  unsigned int i;
+
+  for(i = 0; i < allViews.count(); ++i) {
+    if(allViews.at(i)->isActiveWindow() && allViews.at(i)->getDocument() == this) {
+      curView = allViews.at(i);
+      break;
+    }
+  }
+  if(curView == 0) for(i = 0; i < allViews.count(); ++i) {
+    if(allViews.at(i)->getDocument() == this) {
+      curView = allViews.at(i);
+      break;
+    }
+  }
+
+  ASSERT(curView != 0);
+
+  if(!selectionGroupDialog.isNull()) {
+    if(curView != selectionGroupDialog->parent()) delete selectionGroupDialog;
+    else {
+      selectionGroupDialog->show();
+      selectionGroupDialog->setActiveWindow();
+      return;
+    }
+  }
+
+  selectionGroupDialog = new KSegSelectionGroupDialog(curView, selectionGroups, this);
+
+  selectionGroupDialog->show();
+  selectionGroupDialog->setActiveWindow();
+}
 
 void KSegDocument::doPlay(int which)
 {
@@ -1263,7 +1366,7 @@ void KSegConstruction::constructionRecurse()
     return;
   }
   
-  G_ref *p = new G_ref;
+  G_ref *p = new G_ref();
 
   vector<int> perm;
   G_refs permuted;
@@ -1282,3 +1385,4 @@ void KSegConstruction::constructionRecurse()
 
   emit documentChanged();  
 }
+
